@@ -4,6 +4,9 @@ our $VERSION = $Gtk2::Ex::DbLinker::VERSION;
 
 use strict;
 use warnings;
+
+use Gtk2::Ex::DbLinker::DatasheetHelper;
+use Carp;
 use Glib qw/TRUE FALSE/;
 
 #use Data::Dumper;
@@ -102,7 +105,11 @@ sub new {
     $self->_setup_fields;
 
     $self->_setup_treeview;
-
+    $self->{ds_helper} = Gtk2::Ex::DbLinker::DatasheetHelper->new(
+	   cols => $self->{cols},
+	    dman => $self->{dman},
+	    col_number => sub {$self->colnumber_from_name($_[0])},
+    );
     return $self;
 
 }
@@ -215,6 +222,7 @@ sub _setup_treeview {
     }
 
     # Now set up the model and columns
+
     for my $field ( @{ $self->{fields} } ) {
 
         my $renderer = $render{ $field->{renderer} }[0]();
@@ -259,7 +267,7 @@ sub _setup_treeview {
             #$renderer = $render{$field->{renderer}}();
             # no de la col
             $renderer->{column} = $field->{column};
-
+	    
             if ( $field->{renderer} eq "toggle" ) {
 
                 $renderer->set( activatable => TRUE );
@@ -405,6 +413,7 @@ sub _setup_treeview {
             );
 
         }
+
     }    # for $field ...
 
     #add fields for the combodata if any
@@ -700,6 +709,7 @@ sub set_column_value {
 sub colnumber_from_name {
 
     my ( $self, $fieldname ) = @_;
+    confess ("fieldname undef") unless (defined $fieldname);
     return $self->{colname_to_number}->{$fieldname}
 
 }
@@ -744,6 +754,33 @@ sub _changed {
 }
 
 sub apply {
+    my $self = shift;
+    my $pkref = ( defined $_[0] ? $_[0] : undef );
+   my $model = $self->{treeview}->get_model;
+   my $iter    = $model->get_iter_first;
+   
+   #$self->{ds_helper}->init_iter( $model, $self->{changed_signal});
+   $self->{ds_helper}->init(
+   	sig =>  $self->{changed_signal},
+	sig_block => sub { $self->{log}->debug("sig_block"); $model->signal_handler_block($_[0])},
+   	sig_unblock =>  sub { $model->signal_handler_unblock($_[0]) },
+   	next =>  sub {$model->iter_next($_[0])},
+   	get_val => sub {$self->{log}->debug("get_val col: ", $_[1]); my $x = $model->get($_[0], $_[1]); $self->{log}->debug("get_val found ",  $x); return $x},
+   	set_val => sub {  $self->{log}->debug("set_val", $_[2]); $model->set($_[0], $_[1], $_[2])},
+   	del_row =>  sub  { $model->remove($_[0]) },
+   	has_more_row => sub { return ( defined $iter ? 1 : 0 )},
+    	iter => $iter,
+
+   );
+    #$model->signal_handler_block( $self->{changed_signal} ); 
+    $self->{ds_helper}->apply($pkref);
+    #$model->signal_handler_unblock( $self->{changed_signal} );
+    #return FALSE;
+
+}
+
+
+sub apply_old {
 
     my $self = shift;
     my $pkref = ( defined $_[0] ? $_[0] : undef );
@@ -758,8 +795,11 @@ sub apply {
     my $model = $self->{treeview}->get_model;
 
     my $iter    = $model->get_iter_first;
+    
+    
     my $row_pos = 0;
     my $row;
+
     my @fields_to_save;
     if ($pkref) {
         my %seen;    #remove from cols the fields received in arg
@@ -783,16 +823,12 @@ sub apply {
         # Decide what to do based on status
         if ( $status == UNCHANGED || $status == LOCKED ) {
             $iter = $model->iter_next($iter);
-
-            #$row_pos++;
             next;
         }
 
         if ( $status == INSERTED ) {    # new row for the database
             $self->{dman}->new_row;
-        }    #else { # existing row with CHANGED or DELETED
-             # $row = ${$self->{data}}[$row_pos++];
-             #}
+        }    
 
         if ( $status == DELETED ) {
 
@@ -926,11 +962,20 @@ sub has_changed {
     return ( $self->{changed} ? 1 : 0 );
 }
 
+sub get_current_row {
+	my $self = shift;
+	return $self->{curr_row};
+
+
+}
+
 sub _cell_edited {
     my ( $self, $cell, $path_string, $new_text ) = @_;
+    $self->{log}->debug("_cell_edited path_String : ". $path_string);
     my $path  = Gtk2::TreePath->new_from_string($path_string);
     my $model = $self->{treeview}->get_model;
     my $col   = $cell->{column};
+    $self->{curr_row} = $path_string;
     my $iter  = $model->get_iter($path);
 
     $model->set_value( $iter, $col, $new_text );
@@ -940,6 +985,8 @@ sub _cell_edited {
 sub _toggle_edited {
     my ( $self, $renderer, $text_path, $something ) = @_;
     my $column_no = $renderer->{column};
+    $self->{log}->debug("_toggle_edited path". $text_path);
+    $self->{curr_row} = $text_path;
     my $path      = Gtk2::TreePath->new($text_path);
     my $model     = $self->{treeview}->get_model;
     my $iter      = $model->get_iter($path);
@@ -958,13 +1005,14 @@ sub _combo_edited {
     # return unless ($tree);
     #  treeViewModel[path][columnNumber] = newText
     my $model = $self->{treeview}->get_model;
-    print( "combo_edited " . $new_text . "\n" );
-
+    #print( "combo_edited " . $new_text . "\n" );
+$self->{log}->debug("_combo_edited path_string: ". $path_string);
     #	$cell->get("model");
     #	$model->set ($iter, $cell->{column}, $new_text);
     my $path   = Gtk2::TreePath->new_from_string($path_string);
     my $citer  = $renderer->{combo}->get_active_iter;
     my $cmodel = $renderer->{combo}->get_model;
+    $self->{curr_row} = $path_string;
     my $value  = $cmodel->get( $citer, 0 );
     print( "combo_edited value :" . $value . "\n" );
     my $iter = $model->get_iter($path);
