@@ -4,7 +4,7 @@ our $VERSION = $Gtk2::Ex::DbLinker::VERSION;
 
 use strict;
 use warnings;
-use base 'Gtk2::Ex::DbLinker::AbForm';
+use parent 'Gtk2::Ex::DbLinker::AbForm';
 use Glib qw/TRUE FALSE/;
 use Carp qw(croak confess carp);
 
